/*+
    Name:	vgaregs.c
    Date:	27-Aug-1988
    Author:	Kent J. Quirk
    Abstract:	Contains definitions for VGA hardware and registers.
-*/

#include <conio.h>
#include "vgabios.h"
#include "vgatest.h"
#include "vgaregs.h"

/**** r e a d _ a l l _ g e n e r a l _ r e g s ****
    Abstract:   Reads the contents of all the general registers into a
                predefined structure.
    Parameters: A pointer to the structure
    Returns:    Nothing
****************************/
void read_all_general_regs(GENERAL_REGS *r)
{
    r->m_out  = read_general_reg(MISC_OUT_R);
    r->f_ctrl = read_general_reg(FEATURE_CTRL_R);
    r->istat0 = read_general_reg(INPUT_STATUS_0);
    r->istat1 = read_general_reg(INPUT_STATUS_1);
}

/**** r e a d _ i n d e x e d _ r e g ****
 **** w r i t e _ i n d e x e d _ r e g ****
    Abstract:   Read/write a single VGA indexed register
    Parameters: The register base address, the register read address,
                and the index number (and the value to write).
    Returns:    The byte read.
****************************/
BYTE read_indexed_reg(WORD base, WORD read, BYTE index)
{
    outp(base, index);
    return((BYTE)inp(read));
}

void write_indexed_reg(WORD base, WORD write, BYTE index, BYTE value)
{
    outp(base, index);
    outp(write, value);
}

/**** r e a d _ d a c _ r e g ****
 **** w r i t e _ d a c _ r e g ****
    Abstract:   Read or write a DAC register
    Parameters: The register address (and the byte to write)
    Returns:    The byte read
****************************/
BYTE read_dac_reg(WORD r)
{
    return((BYTE)inp(r));
}

void write_dac_reg(WORD r, BYTE v)
{
    outp(r, v);
    return;
}

/**** r e a d _ a l l _ s e q u e n c e r _ r e g s ****
    Abstract:   Reads all the sequencer registers into a structure.
    Parameters: A pointer to the register structure
    Returns:    Nothing.
****************************/
void read_all_sequencer_regs(SEQUENCER_REGS *r)
{
    BYTE *rp = (BYTE *)r;
    WORD i;
 
    for (i=1; i <= 4; i++)
	rp[i] = (BYTE)read_sequencer_reg(i);
}

/**** r e a d _ a l l _ g r a p h _ r e g s ****
    Abstract:   Reads all the graphics registers into a structure.
    Parameters: A pointer to the register structure.
    Returns:    Nothing
****************************/
void read_all_graph_regs(GRAPHICS_REGS *r)
{
    BYTE *rp = (BYTE *)r;
    WORD i;
 
    for (i=0; i <= 8; i++)
	rp[i] = (BYTE)read_graph_reg(i);
}

/**** r e a d _ a l l _ c r t c _ r e g s ****
    Abstract:   Reads all the CRTC registers into a structure.
    Parameters: A pointer to the register structure.
    Returns:    Nothing
****************************/
void read_all_crtc_regs(CRTC_REGS *r)
{
    BYTE *rp = (BYTE *)r;
    WORD i;
 
    for (i=0; i <= 0x18; i++)
	rp[i] = (BYTE)read_crtc_reg(i);
}

/**** r e a d _ a t t r i b _ r e g ****
 **** w r i t e _ a t t r i b _ r e g ****
    Abstract:   Reads/writes a single attribute register given its index.
    Parameters: The register number (the byte to write)
    Returns:    The byte read (nothing).
****************************/
BYTE read_attrib_reg(WORD r)
{
    inp(INPUT_STATUS_1);	    /* resets address latch */
    outp(ATTR_INDEX, r);
    return((BYTE)inp(ATTR_OTHER_R));
}

void write_attrib_reg(WORD r, BYTE v)
{
    inp(INPUT_STATUS_1);
    outp(ATTR_INDEX, r);	    
    outp(ATTR_OTHER_W, v);
    /* write_indexed_reg(ATTR_INDEX, ATTR_OTHER_W, r, v); */
}

/**** r e a d _ a l l _ a t t r i b _ r e g s ****
    Abstract:   Reads all the attribute registers into a structure.
****************************/
void read_all_attrib_regs(ATTRIB_REGS *r)
{
    BYTE *rp = (BYTE *)r;
    WORD i;

    for (i=0; i <= 5; i++)
	rp[i] = read_attrib_reg(i);
}

/**** w a i t _ f o r _ v e r t i c a l ****
    Abstract:   Waits for a vertical retrace to occur.  
    Parameters: int fast -- if not true, waits for the START of a vertical
                retrace, otherwise just waits until VRETRACE is active.  If
                already in a retrace, this will return immediately (but may
                not leave any time to do useful work without causing snow).
    Returns:    Nothing; only when VRETRACE is active.
****************************/
void wait_for_vertical(int fast)
{
    if (!fast)
	while ((inp(INPUT_STATUS_1) & VRETRACE) == 0)
	    ;						/* wait for retrace */
    while ((inp(INPUT_STATUS_1) & VRETRACE) == VRETRACE)
	;					/* wait for no retrace */
}
